@@ -94,6 +94,7 @@ def read_dm3_eels_info(original_metadata):
                         experiment['exposure_time'] = item
                 if 'frames' in key:
                     experiment['number_of_frames'] = item
+
         if 'Experimental Conditions' in exp_dictionary['EELS']:
             for key, item in exp_dictionary['EELS']['Experimental Conditions'].items():
                 if 'Convergence' in key:
@@ -107,6 +108,14 @@ def read_dm3_eels_info(original_metadata):
             experiment['acceleration_voltage'] = exp_dictionary['Microscope Info']['Voltage']
         if 'Name' in exp_dictionary['Microscope Info']:
             experiment['microscope'] = exp_dictionary['Microscope Info']['Name']
+
+
+    if 'number_of_frames' not in experiment:
+        experiment['number_of_frames'] = 1
+    if 'exposure_time' not in experiment:
+        if 'single_exposure_time' in experiment:
+            experiment['exposure_time'] = experiment['number_of_frames'] * experiment['single_exposure_time']
+
     return experiment
 
 
@@ -472,7 +481,7 @@ def find_edges(dataset, sensitivity=2.5):
     return selected_edges
 
 
-def make_edges(edges_present, energy_scale, e_0, coll_angle):
+def make_edges(edges_present, energy_scale, e_0, coll_angle, low_loss=None):
     """Makes the edges dictionary for quantification
 
     Parameters
@@ -485,6 +494,8 @@ def make_edges(edges_present, energy_scale, e_0, coll_angle):
         acceleration voltage (in V)
     coll_angle: float
         collection angle in mrad
+    low_loss: numpy array with same length as energy_scale
+        low_less spectrum with which to convolve the cross section (default=None)
 
     Returns
     -------
@@ -524,12 +535,12 @@ def make_edges(edges_present, energy_scale, e_0, coll_angle):
         edges[key]['start_exclude'] = edges[key]['onset'] - xsec[edges[key]['symmetry']]['excl before']
         edges[key]['end_exclude'] = edges[key]['onset'] + xsec[edges[key]['symmetry']]['excl after']
 
-    edges = make_cross_sections(edges, energy_scale, e_0, coll_angle)
+    edges = make_cross_sections(edges, energy_scale, e_0, coll_angle, low_loss)
 
     return edges
 
 
-def make_cross_sections(edges, energy_scale, e_0, coll_angle):
+def make_cross_sections(edges, energy_scale, e_0, coll_angle, low_loss=None):
     """Updates the edges dictionary with collection angle-integrated X-ray photo-absorption cross-sections
 
     """
@@ -537,6 +548,11 @@ def make_cross_sections(edges, energy_scale, e_0, coll_angle):
         if key.isdigit():
             edges[key]['data'] = xsec_xrpa(energy_scale, e_0 / 1000., edges[key]['Z'], coll_angle,
                                            edges[key]['chemical_shift']) / 1e10  # from barnes to 1/nm^2
+            if low_loss is not None:
+                low_loss = np.roll(np.array(low_loss), 1024 - np.argmax(np.array(low_loss)))
+                edges[key]['data'] = scipy.signal.convolve(edges[key]['data'], low_loss/low_loss.sum(), mode='same')
+
+
             edges[key]['onset'] = edges[key]['original_onset'] + edges[key]['chemical_shift']
             edges[key]['X_section_type'] = 'XRPA'
             edges[key]['X_section_source'] = 'pyTEMlib'
@@ -1231,7 +1247,11 @@ def xsec_xrpa(energy_scale, e0, z, beta, shift=0):
     xsec = np.zeros(len(energy_scale))
     # shift = 0# int(ek -onsetXRPS)#/dispersion
     lin = interp1d(enexs, datxs, kind='linear')  # Linear instead of spline interpolation to avoid oscillations.
-    xsec = lin(energy_scale - shift)
+    if energy_scale[0] < enexs[0]:
+        start = np.searchsorted(energy_scale, enexs[0])+1
+    else:
+        start = 0
+    xsec[start:] = lin(energy_scale[start:] - shift)
 
     return xsec
 
