@@ -14,34 +14,25 @@ import sys
 
 # For structure files of various flavor for instance POSCAR
 import ase.io
-
+import ipyfilechooser
 
 # =============================================
-#   Include sidpy and other pyTEMlib Libraries                                      #
+#   Include  pycroscopy libraries                                      #
 # =============================================
-from .config_dir import config_path
 
-# from .nsi_reader import NSIDReader
 from SciFiReaders import DM3Reader
 from SciFiReaders import NionReader
 import pyNSID
 import sidpy
 
+# =============================================
+#   Include  pyTEMlib libraries                                      #
+# =============================================
 
+from .config_dir import config_path
 from .sidpy_tools import *
 
-# Open/Save File dialog
-try:
-    from PyQt5 import QtGui, QtWidgets, QtCore
-    QT_available = True
-except ImportError:
-    QT_available = False
-
-if "google.colab" in sys.modules:
-    QT_available = False
-
-if QT_available:
-    from .qt_sidpy_tools import *
+QT_available = False
 
 Dimension = sidpy.Dimension
 nest_dict = sidpy.base.dict_utils.nest_dict
@@ -189,7 +180,13 @@ class FileWidget(object):
 
 def get_qt_app():
     """will start QT Application if not running yet and returns QApplication """
-
+    try:
+        from PyQt5 import QtGui, QtWidgets, QtCore
+        QT_available = True
+    except ImportError:
+        QT_available = False
+        
+    
     # start qt event loop
     _instance = QtWidgets.QApplication.instance()
     if not _instance:
@@ -226,10 +223,17 @@ def save_path(filename):
     return path
 
 
-def savefile_dialog(initial_file='*.hf5', file_types=None):
+def savefile_dialog_Qt(initial_file='*.hf5', file_types=None):
     """Opens a save dialog in QT and returns name of file. New now with initial file"""
 
     # Check whether QT is available
+    """will start QT Application if not running yet and returns QApplication """
+    try:
+        from PyQt5 import QtGui, QtWidgets, QtCore
+        QT_available = True
+    except ImportError:
+        QT_available = False
+        
     if not QT_available:
         print('No QT dialog')
         return None
@@ -253,8 +257,169 @@ def savefile_dialog(initial_file='*.hf5', file_types=None):
     else:
         return ''
 
+class open_file_dialog(ipyfilechooser.FileChooser):
+    def __init__(self, directory=None):
+        if directory == None:
+            directory = get_last_path()
+        super().__init__(directory) 
+        self._use_dir_icons = True
+        
+    def _apply_selection(self):
+        super()._apply_selection()
+        selected = os.path.join(
+            self._selected_path,
+            self._selected_filename
+        )
 
-def openfile_dialog(file_types=None):  # , multiple_files=False):
+        if os.path.isfile(selected):
+            self._label.value = self._LBL_TEMPLATE.format(
+                self._selected_filename,
+                'blue'
+            )
+        else:
+            self._label.value = self._LBL_TEMPLATE.format(
+                self._selected_filename,
+                'green'
+            )
+        
+        save_path(selected)
+        
+    def _set_form_values(self, path: str, filename: str) -> None:
+        """Set the form values."""
+        # Disable triggers to prevent selecting an entry in the Select
+        # box from automatically triggering a new event.
+        self._pathlist.unobserve(
+            self._on_pathlist_select,
+            names='value'
+        )
+        self._dircontent.unobserve(
+            self._on_dircontent_select,
+            names='value'
+        )
+        self._filename.unobserve(
+            self._on_filename_change,
+            names='value'
+        )
+
+        # In folder only mode zero out the filename
+        if self._show_only_dirs:
+            filename = ''
+
+        # Set form values
+        self._pathlist.options = ipyfilechooser.utils.get_subpaths(path)
+        self._pathlist.value = path
+        self._filename.value = filename
+
+        # file/folder real names
+        dircontent_real_names = ipyfilechooser.utils.get_dir_contents(
+            path,
+            show_hidden=self._show_hidden,
+            prepend_icons=False,
+            show_only_dirs=self._show_only_dirs,
+            filter_pattern=self._filter_pattern
+        )
+
+        # file/folder display names
+        dircontent_display_names = ipyfilechooser.utils.get_dir_contents(
+            path,
+            show_hidden=self._show_hidden,
+            prepend_icons=self._use_dir_icons,
+            show_only_dirs=self._show_only_dirs,
+            filter_pattern=self._filter_pattern
+        )
+        dircontent_display_names = self.set_display_names(dircontent_real_names, dircontent_display_names)
+
+        # Dict to map real names to display names
+        self._map_name_to_disp = {
+            real_name: disp_name
+            for real_name, disp_name in zip(
+                dircontent_real_names,
+                dircontent_display_names
+            )
+        }
+
+        # Dict to map display names to real names
+        self._map_disp_to_name = {
+            disp_name: real_name
+            for real_name, disp_name in
+            self._map_name_to_disp.items()
+        }
+
+        # Set _dircontent form value to display names
+        self._dircontent.options = dircontent_display_names
+
+        # If the value in the filename Text box equals a value in the
+        # Select box and the entry is a file then select the entry.
+        if ((filename in dircontent_real_names) and
+                os.path.isfile(os.path.join(path, filename))):
+            self._dircontent.value = self._map_name_to_disp[filename]
+        else:
+            self._dircontent.value = None
+
+        # Reenable triggers again
+        self._pathlist.observe(
+            self._on_pathlist_select,
+            names='value'
+        )
+        self._dircontent.observe(
+            self._on_dircontent_select,
+            names='value'
+        )
+        self._filename.observe(
+            self._on_filename_change,
+            names='value'
+        )
+
+        # Update the state of the select button
+        if self._gb.layout.display is None:
+            # Disable the select button if path and filename
+            # - equal an existing folder in the current view
+            # - equal the already selected values
+            # - don't match the provided filter pattern(s)
+            check1 = filename in dircontent_real_names
+            check2 = os.path.isdir(os.path.join(path, filename))
+            check3 = False
+            check4 = False
+
+            # Only check selected if selected is set
+            if ((self._selected_path is not None) and
+                    (self._selected_filename is not None)):
+                selected = os.path.join(
+                    self._selected_path,
+                    self._selected_filename
+                )
+                check3 = os.path.join(path, filename) == selected
+
+            # Ensure only allowed extensions are used
+            if self._filter_pattern:
+                check4 = not ipyfilechooser.utils.match_item(filename, self._filter_pattern)
+
+            if (check1 and check2) or check3 or check4:
+                self._select.disabled = True
+            else:
+                self._select.disabled = False
+        
+    
+    def set_display_names(self, dircontent_real_names, dircontent_display_names):
+        
+        for i in range(len(dircontent_display_names)):
+            name = dircontent_display_names[i]
+            full_name = os.path.join(self._pathlist.value, dircontent_real_names[i])
+            if os.path.isfile(full_name):
+                size = os.path.getsize(full_name) * 2 ** -20
+                basename, extension = os.path.splitext(name)
+                if extension in ['.hf5']:
+                    dircontent_display_names[i] = f" {dircontent_display_names[i]:50}  -- {size:.1f} MB"
+                
+                elif extension in ['.h5', '.ndata']:
+                    reader = NionReader(full_name)
+                    dataset_nion = reader.read()
+                    dircontent_display_names[i] = f"   {dataset_nion.title+extension:50}  - {size:.1f} MB"
+                else:
+                    dircontent_display_names[i] = dircontent_display_names[i]
+            
+        return dircontent_display_names
+def openfile_dialog_QT(file_types=None):  # , multiple_files=False):
     """Opens a File dialog which is used in open_file() function
 
     This function uses pyQt5.
@@ -281,7 +446,13 @@ def openfile_dialog(file_types=None):  # , multiple_files=False):
     >> print(filename)
 
     """
-
+    """will start QT Application if not running yet and returns QApplication """
+    try:
+        from PyQt5 import QtGui, QtWidgets, QtCore
+        QT_available = True
+    except ImportError:
+        QT_available = False
+        
     # determine file types by extension
     if file_types is None:
         file_types = 'TEM files (*.dm3 *.qf3 *.ndata *.h5 *.hf5);;pyNSID files (*.hf5);;QF files ( *.qf3);;' \
@@ -315,12 +486,7 @@ def save_dataset(dataset, filename=None,  h5_group=None):
         not used yet
 
     """
-    if filename is None:
-        if QT_available:
-            get_qt_app()
-            filename = savefile_dialog()
-        else:
-            raise TypeError('filename must be provided if QT is not installed')
+    filename = open_file_dialog()
     h5_filename = get_h5_filename(filename)
     h5_file = h5py.File(h5_filename, mode='a')
     path, file_name = os.path.split(filename)
@@ -357,7 +523,7 @@ def save_dataset(dataset, filename=None,  h5_group=None):
     return h5_dataset
 
 
-def open_file(filename=None,  h5_group=None):  # save_file=False,
+def open_file(filename=None,  h5_group=None, write_hdf_file=True):  # save_file=False,
     """Opens a file if the extension is .hf5, .ndata, .dm3 or .dm4
 
     If no filename is provided the QT open_file windows opens (if QT_available==True)
@@ -382,11 +548,12 @@ def open_file(filename=None,  h5_group=None):  # save_file=False,
 
     """
     if filename is None:
-        if QT_available:
-            get_qt_app()
-            filename = openfile_dialog()
-        else:
-            raise TypeError('filename must be provided if QT is not installed')
+        selected_file = open_file_dialog()
+        display(selected_file)
+        while selected_file.selected == None:
+            pass
+        file_name = selected_file.selected
+        
     else:
         if not isinstance(filename, str):
             raise TypeError('filename must be a non-empty string or None (to a QT open file dialog)')
@@ -404,6 +571,8 @@ def open_file(filename=None,  h5_group=None):  # save_file=False,
             print('no hdf5 dataset found in file')
             return
         else:
+            if not write_hdf_file:
+                datasets[0].h5_dataset.file.close()
             return datasets[0]
 
         """ should go to no dataset found
@@ -428,21 +597,23 @@ def open_file(filename=None,  h5_group=None):  # save_file=False,
             dset.title = basename.strip().replace('-', '_')
         dset.filename = basename.strip().replace('-', '_')
         # dset.original_metadata = flatten_dict(dset.original_metadata)
-        filename = os.path.join(path,  dset.title+extension)
-        h5_filename = get_h5_filename(filename)
-        h5_file = h5py.File(h5_filename, mode='a')
 
-        if 'Measurement_000' in h5_file:
-            print('could not write dataset to file, try saving it with ft.save()')
-        else:
-            if not isinstance(h5_group, h5py.Group):
-                h5_group = h5_file.create_group('Measurement_000/Channel_000')
-            # dset.axes = dset._axes
-            # dset.attrs = {}
-            h5_dataset = pyNSID.hdf_io.write_nsid_dataset(dset, h5_group)
-            # dset.original_metadata = nest_dict(dset.original_metadata)
+        if write_hdf_file:
+            filename = os.path.join(path,  dset.title+extension)
+            h5_filename = get_h5_filename(filename)
+            h5_file = h5py.File(h5_filename, mode='a')
 
-            dset.h5_dataset = h5_dataset
+            if 'Measurement_000' in h5_file:
+                print('could not write dataset to file, try saving it with ft.save()')
+            else:
+                if not isinstance(h5_group, h5py.Group):
+                    h5_group = h5_file.create_group('Measurement_000/Channel_000')
+                # dset.axes = dset._axes
+                # dset.attrs = {}
+                h5_dataset = pyNSID.hdf_io.write_nsid_dataset(dset, h5_group)
+                # dset.original_metadata = nest_dict(dset.original_metadata)
+
+                dset.h5_dataset = h5_dataset
         return dset
     else:
         print('file type not handled yet.')
@@ -605,7 +776,7 @@ def read_poscar(file_name=None):
     Open file dialog to select a POSCAR file from VASP
     """
     if file_name is None:
-        file_name = openfile_dialog('POSCAR (POSCAR*.txt);;All files (*)')
+        file_name = open_file_dialog('POSCAR (POSCAR*.txt);;All files (*)')
 
     # use ase package to read file
     base = os.path.basename(file_name)
@@ -620,7 +791,7 @@ def read_poscar(file_name=None):
 
 def read_cif(file_name=None, verbose=False):  # open file dialog to select cif file
     if file_name is None:
-        file_name = openfile_dialog('cif (*.cif);;All files (*)')
+        file_name = open_file_dialog('cif (*.cif);;All files (*)')
     # use ase package to read file
 
     base = os.path.basename(file_name)
@@ -647,6 +818,13 @@ def read_cif(file_name=None, verbose=False):  # open file dialog to select cif f
 
 def h5_add_crystal_structure(h5_file, crystal_tags):
     """Write crystal structure to NSID file"""
+    if not isinstance(crystal_tags, dict):
+        try:
+            import ase
+        except ModuleNotFoundError:
+            print('Need a dictionary or an ase.Atoms object with ase installed')
+        if isinstance(crystal_tags, ase.Atoms):
+            return
 
     structure_group = sidpy.hdf.prov_utils.create_indexed_group(h5_file, 'Structure_')
 
